

#include "ap_utils.h"

#include "display_types/viz_point_cloud2.h"
#include "hesai_lidar.h"
#include <sys_ctr.h>
#include <time.h>

HesaiLidar::HesaiLidar()
{
    using namespace ara::com;
    using namespace ara::core;
    using Skeleton = jac::ap::hesaileftlidarservice::skeleton::HesaiLeftLiDARServiceSkeleton;

    // 注意创建Skeleton的第1个参数是InstanceSpecifier类型，其值为配置文件network_binding.json中instanceSpecifier属性的值
    skeleton_ = std::make_shared<Skeleton>(InstanceSpecifier(
        "lidar_hesai_left/lidar_hesai_left/LidarHesaiLeftPPort"),
        MethodCallProcessingMode::kEvent);
}

int HesaiLidar::SetPoints(const hesai::lidar::LidarDecodedFrame<hesai::lidar::LidarPointXYZIRT>  &frame)
{
    // pcXYZIRGlobal.points.clear();
    mdc::visual::PointCloud<mdc::visual::PointXYZIR> pc;
        pc.header.frameId = "iv_points";
        for (uint32_t i = 1; i < frame.points_num; i++) {
            mdc::visual::PointXYZIR p(frame.points[i].x, frame.points[i].y, frame.points[i].z, frame.points[i].intensity, (uint32_t)frame.points[i].ring);
            //printf("frame:%d x:%lf y:%lf z:%lf \n",frame.frame_index, frame.points[i].x,frame.points[i].y,frame.points[i].z);
            pc.points.push_back(p);
        }
        pc.isDense = true;
        pointsQueue.push(pc);
        //printf("length pcXYZIRGlobal:%d \n",pcXYZIRGlobal.points.size());
        return 0;
}


void HesaiLidar::Publish(mdc::visual::PointCloud<mdc::visual::PointXYZIR> pc)
{
    auto pointCloud = skeleton_->HesaiLeftLiDAR2LiDARPerceptionEvent.Allocate();
    //mdc::visual::PointCloud<mdc::visual::PointXYZIR> pcToSend;
	perception::PointXYZIT p;
    //mdc::visual::PointCloud<mdc::visual::PointXYZIR> pc;

	pointCloud->header.sensor_name = 3;
	for (uint32_t i = 1; i < pc.points.size(); i++)
	{
		p.intensity = pc.points[i].intensity;
		p.x = pc.points[i].x;
		p.y = pc.points[i].y;
		p.z = pc.points[i].z;
		pointCloud->points.push_back(p);
	}
        timespec ts;
	    clock_gettime(CLOCK_VIRTUAL, &ts);
        pointCloud->header.stampHigh = ts.tv_sec;
        pointCloud->header.stampLow = ts.tv_nsec;
	 //发送Event数据包
    skeleton_->HesaiLeftLiDAR2LiDARPerceptionEvent.Send(std::move(pointCloud));
}


void HesaiLidar::sendPointCloud()
{
    // if (width){
    //     pcPointXYZIRPub.Publish(pcXYZIRGlobal);
    // }
    if(!pointsQueue.empty())
    {
    	Publish(pointsQueue.front());
        //pcPointXYZIRPub.Publish(pointsQueue.front());
        pointsQueue.pop();
         printf("[SUCCESS] Published Hesai Successfully and queue size is = %d\n", pointsQueue.size());
    }
    
}

void HesaiLidar::Init()
{
    skeleton_->OfferService();
}


bool HesaiLidar::OnInit()
{
    //pcPointXYZIRPub = mdc::visual::Publisher::Advertise<mdc::visual::PointCloud2>(ara::core::String("hesai_lidar_adib"));

    return 1;
}

void HesaiLidar::setReady()
{
    ready = true;
}

bool HesaiLidar::isReady()
{
    return ready;
}

void HesaiLidar::OnSend()
{
    if(ready) sendPointCloud();
    // printf("[LOG] HesaiLidar on send\n");

}

void HesaiLidar::RunThread()
{
    while(1)
    {
        OnSend();
        std::this_thread::sleep_for(std::chrono::milliseconds(10));
    }
}

